
//#include "metalShaderTypes.h"

typedef struct
{
	int direction;
} fxVars;

constant vec2 fsDirectionTable[9] = {
	vec2(1,-.5),
	vec2(0,-.5),
	vec2(-1,-.5),
	vec2(1,0),
	vec2(0,0),
	vec2(-1,0),
	vec2(1,.5),
	vec2(0,.5),
	vec2(-1,.5)
};

constant vec2 center = vec2(0.5, 0.5);


vertex vertexOut vertexFunc(uint vertexID [[ vertex_id ]],const device txShaderVers* in[[ buffer(0) ]],constant txShaderUniforms& uniforms[[ buffer(3) ]],constant fxVars& vars[[ buffer(2)]])
{
    vertexOut out;

    out.position=in[vertexID].pos*uniforms.worldViewProj;

    out.tex1=in[vertexID].tex1;
	out.tex2=in[vertexID].tex2;
	out.texMask=in[vertexID].texMask;
	out.normPos=in[vertexID].pos.xy*vec2(.5,.5f)+vec2(.5,.5);
    return out;
}

fragment float4 fragmentFunc(vertexOut in [[stage_in]]
							 ,texture2d<half> inputTex0 [[ texture(0) ]]
							 ,texture2d<half> inputTex1 [[ texture(1) ]]
							 ,texture2d<half> inputTexMask [[ texture(2) ]]
							 ,constant txShaderUniforms& u[[ buffer(2) ]]
							 ,constant fxVars& vars[[ buffer(1)]])
{
	vec2 fsDirection=normalize(fsDirectionTable[vars.direction]);
	vec2 p = in.normPos;
	vec2 v = fsDirection;
	if(v.x==0.0)
		v.x+=.01;
	if(v.y==0.0)
		v.y+=.01;

	v /= abs(v.x)+abs(v.y);
	float d = v.x * center.x + v.y * center.y;
	float offset = 1.0;
	float pr = smoothstep(-offset, 0.0, v.x * p.x + v.y * p.y - (d-0.5+u.transitionProgressLinear*(1.+offset)));
	vec2 squarep = fract(p*vec2(10,10));
	vec2 squaremin = vec2(pr/2.0);
	vec2 squaremax = vec2(1.0 - pr/2.0);
	float a = all(squaremin<squarep) && all(squarep<squaremax) ? 1.0 : 0.0;
	vec4 texelA=tex(inputTex0,in.tex1);
	vec4 outColorA=mix(vec4(0,0,0,0),texelA,a);
	vec4 outColorB=mix(texelA,vec4(0,0,0,0),a);
	float4 outColor=mix(outColorB,outColorA,u.mix);

#ifdef USE_MASK
	outColor.a*=float(inputTexMask.sample(linearSampler,in.texMask).a);
#endif

	return outColor;
}
